#!/bin/bash

set -e

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [xray.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

MAX_OPEN_FILES_MIN_VALUE=10000
MAX_OPEN_FILES_REQUIRED_VALUE=32000

# XRAY_APP_SERVICES_LIST, the services names as they are in the docker-cmpose file, SPACE separated
XRAY_APP_SERVICES_LIST="xray-server xray-indexer xray-analysis xray-event xray-persist"

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [xray.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

aparse()
{
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    while [[ $# > 0 ]] ; do
      case "$1" in
        --compose-flags)
            shift
              export DOCKER_COMPOSE_FLAGS=${1}
              shift
          ;;
        --server-flags)
              shift
              export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=${1}
              shift
          ;;
        --persist-flags)
              shift
              export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=${1}
              shift
          ;;
        --indexer-flags)
              shift
              export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=${1}
              shift
          ;;
        --xevent-flags)
              shift
              export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=${1}
              shift
          ;;
        --analysis-flags)
              shift
              export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=${1}
              shift
          ;;
          *)
              ALL_ARGS="${ALL_ARGS}${1} "
              shift
          ;;
      esac
    done
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="xray"
fi

# Installed version
INSTALLER_DATA_FOLDER="/data"
INSTALLED_VERSION_FILE="${INSTALLER_DATA_FOLDER}/version.current"
INSTALLED_VERSION_FILE_HISTORY="${INSTALLER_DATA_FOLDER}/version.history"
INSTALLED_VERSION=''
EULA_DOC="${INSTALLER_DATA_FOLDER}/JFrog_Xray_EULA.info"
EULA_DOC_SRC="/opt/jfrog/xray/installer/JFrog_Xray_EULA.info"
XRAY_GLOBAL_MOUNT_ROOT="/xray_global_mount_root" # This should not be used, unless special maintenance is needed

if [ -f $INSTALLED_VERSION_FILE ]
then
    INSTALLED_VERSION=$(cat $INSTALLED_VERSION_FILE)
fi

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="/opt/jfrog/xray/installer/version.current"
MAINTAINER_VERSION=''
if [ -f $MAINTAINER_VERSION_FILE ]
then
    MAINTAINER_VERSION=$(cat $MAINTAINER_VERSION_FILE)
fi

#load xray configuration
XRAY_SERVER_PORT_DEFAULT=8000
XRAY_SERVER_PORT=$(./getXrayConfValue.py -f "${XRAY_GLOBAL_MOUNT_ROOT}/xray/config/xray_config.yaml" -k "XrayServerPort" 2>/dev/null ) || true #echo "XrayServerPort cannot be loaded from xray_config.yaml. Using default value: ${XRAY_SERVER_PORT_DEFAULT} "
XRAY_SERVER_PORT=${XRAY_SERVER_PORT:-${XRAY_SERVER_PORT_DEFAULT}}
re_is_num='^[0-9]+$'
if ! [[ $XRAY_SERVER_PORT =~ $re_is_num ]] ; then
   echo "ERROR: xray_config.yaml: XrayServerPort is not a number" >&2; exit 1
fi

# Default compose env
export XRAY_SERVER_PORT=${XRAY_SERVER_PORT}
export XRAY_DATA="/var/opt/jfrog/xray/data"
export DOCKER_COMPOSE_FLAGS=''
export DOCKER_COMPOSE_XRAY_SERVER_FLAGS=''
export DOCKER_COMPOSE_XRAY_PERSIST_FLAGS=''
export DOCKER_COMPOSE_XRAY_INDEXER_FLAGS=''
export DOCKER_COMPOSE_XRAY_XEVENT_FLAGS=''
export DOCKER_COMPOSE_XRAY_ANALYSIS_FLAGS=''

exitOnError()
{
    msg=$1
    echo "ERROR: ${msg}"
    exit 1
}

usage() {
    cat ./readme.txt
    exit 1
}

version_lt() { test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1"; }

# $1 should be WARNING or ERROR
printUlimitErrorMsg()
{
    local error_level=$1
    local number=$2
    MSG="
${error_level}: Max number of open files [${number}] is too low !!!

Add the following to /etc/security/limits.conf file on HOST:
* soft nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}
* hard nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}

Note: Also verify docker limits configuration if exists [usually in /etc/default/docker]

"

    case ${error_level} in
    "ERROR")
        echo -e "\033[31m${MSG}\033[0m "
    ;;
    "WARNING")
        echo -e "\033[33m${MSG}\033[0m"
    ;;
    esac

    echo
}


checkPrerequisites()
{
    #echo "Check dependencies:"

    if [ -z ${DOCKER_SERVER_CURRENT_VER} ] || [ "${DOCKER_SERVER_CURRENT_VER}" == "" ]
    then
        echo "WARNING: Docker daemon version is unknown"
        # Do not exit on warning
    fi

    if version_lt ${DOCKER_SERVER_CURRENT_VER} ${DOCKER_SERVER_MIN_VER}
    then
        echo ""
        exitOnError "Xray requires Docker daemon version ${DOCKER_SERVER_MIN_VER} or later. The current version is ${DOCKER_SERVER_CURRENT_VER}."
    fi

    # echo "Check max number of open files:"
    currentMaxOpenFilesVal=$(ulimit -n)

    if [ "${currentMaxOpenFilesVal}" != "unlimited" ] ; then

        # MAX_OPEN_FILES_REQUIRED_VALUE
        if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_REQUIRED_VALUE} ]
        then
            #echo "Trying to set max open files = ${MAX_OPEN_FILES_REQUIRED_VALUE}"
            ulimit -n ${MAX_OPEN_FILES_REQUIRED_VALUE} 2>/dev/null
            if [ $? -gt 0 ]
            then
                # Cannot set MAX_OPEN_FILES_REQUIRED_VALUE, will try to set MAX_OPEN_FILES_MIN_VALUE
                # MAX_OPEN_FILES_MIN_VALUE
                if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_MIN_VALUE} ]
                then
                    #echo "Trying to set max open files = ${MAX_OPEN_FILES_MIN_VALUE}"
                    ulimit -n ${MAX_OPEN_FILES_MIN_VALUE} 2>/dev/null
                    if [ $? -gt 0 ]; then
                        printUlimitErrorMsg "ERROR" "${currentMaxOpenFilesVal}"
                        return 1
                    fi
                fi
                printUlimitErrorMsg "WARNING" "${currentMaxOpenFilesVal}"
                # Do not exit on warning
            fi
        fi

    fi

    # dependencies check is passed
    return 0
}

runDockerCommand() {
    local action=$1

    case $action in
    "pull")
       if  [ "$DOCKER_INSTALLER_OFFLINE" == "true" ]; then
            return 0;
       fi
    ;;
    "logs")
        action="logs -f"
    ;;
    "stopApp")
        # Stop Xray App layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${XRAY_APP_SERVICES_LIST}
        return $?
    ;;
    "restart")
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop"
        sleep 3
        action="up -d"
    ;;
    "restartApp")
        # Restart Xray App Layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${XRAY_APP_SERVICES_LIST}
        sleep 3
        action="up -d"
    ;;
    esac

    docker-compose -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {

    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    if [ -z $MAINTAINER_VERSION ]
    then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z $INSTALLED_VERSION ]
    then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]
    then
        installedVersion=($(echo $INSTALLED_VERSION | sed "s/^b//g"))
        maintainerVersion=($(echo $MAINTAINER_VERSION | sed "s/^b//g"))
    fi

    verComp $installedVersion $maintainerVersion 2>/dev/null
    if [ $? -eq 1 ]
    then
        return 1
    fi

    return 0
}

isCommandAllowed()
{
    local command=$1
    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    case ${command} in
        "install")
            [ -f $INSTALLED_VERSION_FILE ] && exitOnError "Xray is already installed. To upgrade, use: ./xray upgrade"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
        ;;
        "upgrade")
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Xray is not installed. To install, use: ./xray install"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
            isUpgradeAllowed || exitOnError "Unable to upgrade to an older version"
        ;;
        "version")

        ;;
        "info")

        ;;
        *)
            # Is installed
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Xray is not installed. To install, use: ./xray install"
            # Is correct version
            [ "${installedVersion}" == "${maintainerVersion}" ] || exitOnError "You are using the Xray installer version [${maintainerVersion}] which is different from the currently installed version [${installedVersion}]. Consider upgrading your version of Xray using: ./xray upgrade"
        ;;

    esac

    return 0
}

setVersions(){

    if [ ! -f ${MAINTAINER_VERSION_FILE} ]
    then
        return 1
    fi

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

setEULA(){
    cp ${EULA_DOC_SRC} ${EULA_DOC}
    return $?
}

createMongoUsers(){
    USERS_CREATED_FILE="${INSTALLER_DATA_FOLDER}/mongo_users.created"
    if [ ! -f "${USERS_CREATED_FILE}" ];then
      sleep 1
      cat createMongoUsers.js | docker exec -i "${COMPOSE_PROJECT_NAME}_mongodb_1" mongo  > /dev/null && \
      touch $USERS_CREATED_FILE || exitOnError "Failed to create MongoDB default users"
    fi
}

# backward compatible issues
# Xray should be stopped before this process
backwardCmpatibleProcess()
{
    #echo -e "\nINFO: Backward compatible process"
    LOGS_FOLDER_V1="${XRAY_GLOBAL_MOUNT_ROOT}/xray/log"
    LOGS_FOLDER_V2="${XRAY_GLOBAL_MOUNT_ROOT}/xray/logs"
    CONFIG_FOLDER="${XRAY_GLOBAL_MOUNT_ROOT}/xray/config"
    ARCHIVE_FOLDER="${INSTALLER_DATA_FOLDER}/archive"
    TIMESTAMP=$(date +"%s")

    # Rename log folder to be logs
    if [ -d "${LOGS_FOLDER_V1}" ] && [ ! -d "${LOGS_FOLDER_V2}" ]
    then
        mv ${LOGS_FOLDER_V1} ${LOGS_FOLDER_V2} && \
        echo "Logs folder renamed was:[${LOGS_FOLDER_V1}] new:[${LOGS_FOLDER_V2}]"

        # Archive old log4go_config.xml
        if [ ! -d ${ARCHIVE_FOLDER} ]; then
            mkdir -p ${ARCHIVE_FOLDER} && \
            echo "Archive folder was created [${ARCHIVE_FOLDER}]"
        fi

        for log4goService in "event" "analysis" "indexer" "persist" "server"
        do
           echo "Archive ${log4goService}_log4go_config.xml to ${ARCHIVE_FOLDER}"
           if [ -f "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" ] && [ -d "${ARCHIVE_FOLDER}" ]
           then
                mv "${CONFIG_FOLDER}/${log4goService}_log4go_config.xml" "${ARCHIVE_FOLDER}/${log4goService}_log4go_config.xml.${TIMESTAMP}"
           fi
        done

    fi
}
##################################
############## MAIN ##############
##################################

aparse $@

isCommandAllowed ${command}

case $command in
    install)
        ( runDockerCommand pull && \
          setVersions && \
          setEULA \
        ) || exitOnError "Installation failed"
        echo
        echo "INFO: Xray $MAINTAINER_VERSION was successfully installed. Run: ./xray start"
        echo "To access Xray, browse to http://<server_name>:8000"
        echo "For more information, please refer to the JFrog Xray User Guide: https://www.jfrog.com/confluence/display/XRAY/Welcome+to+JFrog+Xray"
        ;;
    upgrade)
        ( runDockerCommand pull && \
          runDockerCommand stopApp && \
          backwardCmpatibleProcess && \
          setVersions
        ) || exitOnError "Upgrade failed"
        echo
        echo "INFO: Xray was successfully upgraded. Run: ./xray start"
        echo ""
        ;;
    start)
        runDockerCommand "up -d" && \
        createMongoUsers
        ;;
    ps|logs|stop|kill|restart|restartApp)
        runDockerCommand ${command}
        ;;
    x-rm)
        runDockerCommand rm
        ;;
    version)
        if [ -f $INSTALLED_VERSION_FILE ]; then
            echo "Xray version: ${INSTALLED_VERSION}"
        else
            echo "Xray is not installed"
        fi
        ;;
    x-compose)
            xComposeCmd="docker-compose -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
            echo "Advanced user: Going to execute '${xComposeCmd}'"
            ${xComposeCmd}
        ;;
    info)
           echo "Xray info:"
           echo "server port: ${XRAY_SERVER_PORT}"
           echo "project name: ${COMPOSE_PROJECT_NAME}"
           echo "Xray docker repo: ${XRAY_DOCKER_REPO}"
           echo "Xray DB docker repo: ${XRAYDB_DOCKER_REPO}"
           echo "docker mount folder: ${XRAY_MOUNT_ROOT}"
        ;;
    *)
        usage
        ;;
esac